@@ -35,19 +35,54 @@ from gst.pbutils import INSTALL_PLUGINS_SUCCESS, \
         INSTALL_PLUGINS_PARTIAL_SUCCESS, INSTALL_PLUGINS_USER_ABORT, \
         INSTALL_PLUGINS_STARTED_OK
 import tempfile
-from base64 import urlsafe_b64encode
+import hashlib
 
 from pitivi.log.loggable import Loggable
 from pitivi.factories.file import FileSourceFactory, PictureFileSourceFactory
 from pitivi.stream import get_stream_for_pad
 from pitivi.signalinterface import Signallable
-from pitivi.stream import VideoStream
+from pitivi.stream import VideoStream, TextStream
+from pitivi.settings import xdg_cache_home
 
 # FIXME: We need to store more information regarding streams
 # i.e. remember the path took to get to a raw stream, and figure out
 # what encoded format it is
 # We will need that in order to create proper Stream objects.
 
+class EOSSir(gst.Element):
+    __gstdetails__ = (
+        "EOSSir",
+        "Generic",
+        "pushes EOS after the first buffer",
+        "Alessandro Decina <alessandro.d@gmail.com>"
+        )
+
+    srctemplate = gst.PadTemplate("src", gst.PAD_SRC,
+            gst.PAD_ALWAYS, gst.Caps("ANY"))
+    sinktemplate = gst.PadTemplate("sink", gst.PAD_SINK,
+            gst.PAD_ALWAYS, gst.Caps("ANY"))
+
+    __gsttemplates__ = (srctemplate, sinktemplate)
+
+    def __init__(self):
+        gst.Element.__init__(self)
+
+        self.sinkpad = gst.Pad(self.sinktemplate, "sink")
+        self.sinkpad.set_chain_function(self.chain)
+        self.add_pad(self.sinkpad)
+
+        self.srcpad = gst.Pad(self.srctemplate, "src")
+        self.add_pad(self.srcpad)
+
+    def chain(self, pad, buf):
+        ret = self.srcpad.push(buf)
+        if ret == gst.FLOW_OK:
+            self.info("pushed, doing EOS")
+            self.srcpad.push_event(gst.event_new_eos())
+
+        return ret
+gobject.type_register(EOSSir)
+
 class Discoverer(Signallable, Loggable):
     """
     Queues requests to discover information about given files.
@@ -90,6 +125,7 @@ class Discoverer(Signallable, Loggable):
         self.error = None
         self.error_detail = None
         self.unfixed_pads = 0
+        self.unknown_pads = 0
         self.missing_plugin_messages = []
         self.dynamic_elements = []
         self.thumbnails = {}
@@ -103,9 +139,9 @@ class Discoverer(Signallable, Loggable):
             self.bus = None
 
         if self.pipeline is not None:
-            self.info("before setting to NULL")
+            self.debug("before setting to NULL")
             res = self.pipeline.set_state(gst.STATE_NULL)
-            self.info("after setting to NULL : %s", res)
+            self.debug("after setting to NULL : %s", res)
 
         for element in self.dynamic_elements:
             self.pipeline.remove(element)
@@ -184,6 +220,8 @@ class Discoverer(Signallable, Loggable):
         self._finishAnalysisAfterResult(rescan=rescan)
 
     def _emitError(self):
+        self.debug("emitting error %s, %s, %s",
+                self.current_uri, self.error, self.error_detail)
         self.emit("discovery-error", self.current_uri, self.error, self.error_detail)
 
     def _emitErrorMissingPlugins(self):
@@ -257,7 +295,7 @@ class Discoverer(Signallable, Loggable):
         # self._installMissingPluginsCallback is called by the application
         return False
 
-    def _finishAnalysis(self):
+    def _finishAnalysis(self, reason):
         """
         Call this method when the current file is analyzed
         This method will wrap-up the analyzis and call the next analysis if needed
@@ -265,6 +303,8 @@ class Discoverer(Signallable, Loggable):
         if self.timeout_id:
             self._removeTimeout()
 
+        self.info("analysys finished, reason %s", reason)
+
         # check if there are missing plugins before calling _resetPipeline as we
         # are going to pop messagess off the bus
         self._checkMissingPlugins()
@@ -294,7 +334,7 @@ class Discoverer(Signallable, Loggable):
         if not self.error:
             self.error = _('Timeout while analyzing file.')
             self.error_detail = _('Analyzing the file took too long.')
-        self._finishAnalysis()
+        self._finishAnalysis("timeout")
 
         return False
 
@@ -344,6 +384,7 @@ class Discoverer(Signallable, Loggable):
             dbin = gst.element_factory_make("decodebin", "dbin")
 
         dbin.connect("new-decoded-pad", self._newDecodedPadCb)
+        dbin.connect("unknown-type", self._unknownType)
 
         return dbin
 
@@ -369,7 +410,7 @@ class Discoverer(Signallable, Loggable):
         # create the source element
         source = self._createSource()
         if source is None:
-            self._finishAnalysis()
+            self._finishAnalysis("no source")
             return False
 
         # create decodebin(2)
@@ -389,7 +430,7 @@ class Discoverer(Signallable, Loggable):
             if not self.error:
                 self.error = _("Pipeline didn't want to go to PAUSED.")
             self.info("Pipeline didn't want to go to PAUSED")
-            self._finishAnalysis()
+            self._finishAnalysis("failure going to PAUSED")
 
             return False
 
@@ -399,9 +440,9 @@ class Discoverer(Signallable, Loggable):
         return False
 
     def _busMessageEosCb(self, unused_bus, message):
-        self.log("got EOS")
+        self.debug("got EOS")
 
-        self._finishAnalysis()
+        self._finishAnalysis("EOS")
 
     def _busMessageErrorCb(self, unused_bus, message):
         gerror, detail = message.parse_error()
@@ -413,7 +454,7 @@ class Discoverer(Signallable, Loggable):
         self.error = _("An internal error occurred while analyzing this file: %s") % gerror.message
         self.error_detail = detail
 
-        self._finishAnalysis()
+        self._finishAnalysis("ERROR")
 
     def _busMessageElementCb(self, unused_bus, message):
         self.debug("Element message %s", message.structure.to_string())
@@ -423,7 +464,7 @@ class Discoverer(Signallable, Loggable):
                 self.error = _("File contains a redirection to another clip.")
                 self.error_detail = _("PiTiVi currently does not handle redirection files.")
 
-            self._finishAnalysis()
+            self._finishAnalysis("redirect")
             return
 
         if gst.pbutils.is_missing_plugin_message(message):
@@ -443,13 +484,17 @@ class Discoverer(Signallable, Loggable):
         if prev == gst.STATE_READY and new == gst.STATE_PAUSED and \
                 pending == gst.STATE_VOID_PENDING:
             have_video, have_audio, have_image = self._getCurrentStreamTypes()
-            if self.unfixed_pads or have_video or have_image:
+            if self.unfixed_pads or self.unknown_pads or have_video or have_image:
                 # go to PLAYING to generate the thumbnails
-                self.pipeline.set_state(gst.STATE_PLAYING)
+                if self.pipeline.set_state(gst.STATE_PLAYING) == gst.STATE_CHANGE_FAILURE:
+                    if not self.error:
+                        self.error = _("Pipeline didn't want to go to PLAYING.")
+                    self.info("Pipeline didn't want to go to PAUSED")
+                    self._finishAnalysis("failure going to PAUSED")
             elif self.unfixed_pads == 0:
                 # check for unfixed_pads until elements are fixed to do
                 # negotiation before pushing in band data
-                self._finishAnalysis()
+                self._finishAnalysis("got to PAUSED and no unfixed pads")
 
     def _busMessageTagCb(self, unused_bus, message):
         self.debug("Got tags %s", message.structure.to_string())
@@ -471,17 +516,102 @@ class Discoverer(Signallable, Loggable):
         return tmp
 
     def _getThumbnailFilenameFromPad(self, pad):
-        tmp = self._gettempdir()
-        name = '%s.%s' % (self.current_uri, pad.get_name())
-        name = urlsafe_b64encode(name) + '.png'
-        filename = os.path.join(tmp, name)
+        base = xdg_cache_home()
+        name = self.current_uri
+        md5sum = hashlib.md5()
+        md5sum.update(self.current_uri)
+        name = md5sum.hexdigest() + '.png'
+        directory = os.path.join(base, "pitivi")
+        try:
+            os.makedirs(directory)
+        except OSError, e:
+            # 17 = file exists
+            if e.errno != 17:
+                raise
+        filename = os.path.join(base, "pitivi", name)
 
         return filename
 
+    def _videoPadSeekCb(self, pad):
+        try:
+            duration = self.pipeline.query_duration(gst.FORMAT_TIME)[0]
+        except gst.QueryError:
+            duration = 0
+
+        self.debug("doing thumbnail seek at %s", gst.TIME_ARGS(duration))
+
+        if duration:
+            self.pipeline.seek_simple(gst.FORMAT_TIME,
+                    gst.SEEK_FLAG_FLUSH, duration / 3)
+
+        pad.set_blocked_async(False, self._videoPadBlockCb)
+
+    def _videoPadBlockCb(self, pad, blocked):
+        self.debug("video pad blocked: %s" % blocked)
+        if blocked:
+            gobject.timeout_add(0, self._videoPadSeekCb, pad)
+
+    def _addVideoBufferProbe(self, pad):
+        closure = {}
+        closure['probe_id'] = pad.add_buffer_probe(self._videoBufferProbeCb,
+                closure)
+
+    def _removeVideoBufferProbe(self, pad, closure):
+        pad.remove_buffer_probe(closure['probe_id'])
+
+    def _videoBufferProbeCb(self, pad, buf, closure):
+        self.log("video buffer probe for pad %s", pad)
+        self._removeVideoBufferProbe(pad, closure)
+
+        pad.set_blocked_async(True, self._videoPadBlockCb)
+
+        return False
+
+    def _padEventProbeCb(self, pad, event):
+        self.log("got event %s from src %s on pad %s",
+                event.type, event.src, pad)
+
+        return True
+
+    def _padBufferProbeCb(self, pad, buf):
+        self.debug("got buffer on pad %s", pad)
+
+        return True
+
+    def _addPadProbes(self, pad):
+        pad.add_event_probe(self._padEventProbeCb)
+        pad.add_buffer_probe(self._padBufferProbeCb)
+
     def _newVideoPadCb(self, pad):
         """ a new video pad was found """
         self.debug("pad %r", pad)
 
+        self._addPadProbes(pad)
+
+        thumbnail = self._getThumbnailFilenameFromPad(pad)
+        self.thumbnails[pad] = thumbnail
+        have_thumbnail = os.path.exists(thumbnail)
+
+        if have_thumbnail:
+            self.debug("we already have a thumbnail %s for %s", thumbnail, pad)
+            sink = gst.element_factory_make("fakesink")
+            # use this and not fakesink.props.num_buffers = 1 to avoid some
+            # not-expected errors when discovering pictures
+            eossir = EOSSir()
+            self.dynamic_elements.extend([eossir, sink])
+            self.pipeline.add(eossir, sink)
+            eossir.set_state(gst.STATE_PLAYING)
+            sink.set_state(gst.STATE_PLAYING)
+
+            pad.link(eossir.get_pad("sink"))
+            eossir.link(sink)
+
+            return
+
+        stream = get_stream_for_pad(pad)
+        if isinstance(stream, VideoStream) and not stream.is_image:
+            self._addVideoBufferProbe(pad)
+
         queue = gst.element_factory_make("queue")
         queue.props.max_size_bytes = 5 * 1024 * 1024
         queue.props.max_size_time = 5 * gst.SECOND
@@ -491,7 +621,6 @@ class Discoverer(Signallable, Loggable):
         pngenc = gst.element_factory_make("pngenc")
         pngenc.props.snapshot = True
         pngsink = gst.element_factory_make("filesink")
-        self.thumbnails[pad] = thumbnail = self._getThumbnailFilenameFromPad(pad)
         pngsink.props.location = thumbnail
 
         self.dynamic_elements.extend([queue, vscale, csp, pngenc, pngsink])
@@ -506,8 +635,16 @@ class Discoverer(Signallable, Loggable):
             element.sync_state_with_parent()
 
     def _newPadCb(self, pad):
+        stream = get_stream_for_pad(pad)
+        if isinstance(stream, TextStream):
+            self.info("skipping subtitle pad")
+            return
+
+        self._addPadProbes(pad)
+
         queue = gst.element_factory_make('queue')
         fakesink = gst.element_factory_make('fakesink')
+        fakesink.props.num_buffers = 1
         self.dynamic_elements.append(queue)
         self.dynamic_elements.append(fakesink)
 
@@ -523,12 +660,16 @@ class Discoverer(Signallable, Loggable):
             ghost = pad
 
         caps = pad.props.caps
+        self.debug("pad caps notify %s", caps)
         if caps is None or not caps.is_fixed():
             return
 
         pad.disconnect_by_func(self._capsNotifyCb)
 
+        self.info("got fixed caps for pad %s", pad)
+
         self.unfixed_pads -= 1
+        self.debug("unfixed pads %d", self.unfixed_pads)
         stream = self._addStreamFromPad(ghost)
         if isinstance(stream, VideoStream):
             stream.thumbnail = self.thumbnails[ghost]
@@ -543,13 +684,13 @@ class Discoverer(Signallable, Loggable):
             self._newPadCb(pad)
 
         # try to get the duration
-        # NOTE: this gets the duration only once, usually for the first stream.
-        # Demuxers don't seem to implement per stream duration queries anyway.
         self._maybeQueryDuration(pad)
 
         caps = pad.props.caps
 
         if caps is not None and caps.is_fixed():
+            self.debug("got fixed caps for pad %s", pad)
+
             stream = self._addStreamFromPad(pad)
             if isinstance(stream, VideoStream):
                 stream.thumbnail = self.thumbnails[pad]
@@ -565,8 +706,16 @@ class Discoverer(Signallable, Loggable):
             else:
                 pad.connect("notify::caps", self._capsNotifyCb)
             self.unfixed_pads += 1
+            self.debug("unfixed pads %d", self.unfixed_pads)
+
+    def _unknownType(self, decodebin, pad, caps):
+        # decodebin2 sends ASYNC_DONE when it finds an unknown type so we have
+        # to deal with that...
+        self.unknown_pads += 1
 
     def _addStreamFromPad(self, pad):
+        self._maybeQueryDuration(pad)
+        self.debug("adding stream from pad %s caps %s", pad, pad.props.caps)
         stream  = get_stream_for_pad(pad)
         self.current_streams.append(stream)
 
