@@ -27,8 +27,10 @@ import gst
 
 from pitivi.log.loggable import Loggable
 from pitivi.elements.singledecodebin import SingleDecodeBin
+from pitivi.elements.smartscale import SmartVideoScale
 from pitivi.signalinterface import Signallable
-from pitivi.stream import match_stream_groups_map, AudioStream
+from pitivi.stream import match_stream_groups_map, AudioStream, VideoStream
+from pitivi.utils import formatPercent
 
 # FIXME: define a proper hierarchy
 class ObjectFactoryError(Exception):
@@ -208,6 +210,8 @@ class SourceFactory(ObjectFactory):
         'bin-released': ['bin']
     }
 
+    ffscale_factory = 'ffvideoscale'
+
     # make this an attribute to inject it from tests
     singleDecodeBinClass = SingleDecodeBin
 
@@ -217,6 +221,17 @@ class SourceFactory(ObjectFactory):
         self.uri = uri
         self.max_bins = -1
         self.current_bins = 0
+        self._filtercaps = gst.Caps("video/x-raw-rgb;video/x-raw-yuv")
+
+    def getInterpolatedProperties(self, stream):
+        self.debug("stream:%r", stream)
+        props = ObjectFactory.getInterpolatedProperties(self, stream)
+        if isinstance(stream, AudioStream):
+            props.update({"volume" : (0.0, 2.0, formatPercent)})
+        elif isinstance(stream, VideoStream):
+            props.update({"alpha" : (0.0, 1.0, formatPercent)})
+        self.debug("returning %r", props)
+        return props
 
     def makeBin(self, output_stream=None):
         """
@@ -318,6 +333,9 @@ class SourceFactory(ObjectFactory):
             bin.decodebin.disconnect_by_func(self._singlePadRemovedCb)
 
         del bin.decodebin
+        if hasattr(bin, "child"):
+            bin.child.set_state(gst.STATE_NULL)
+            del bin.child
 
         if hasattr(bin, "volume"):
             # only audio bins have a volume element
@@ -327,6 +345,13 @@ class SourceFactory(ObjectFactory):
             del bin.volume
             del bin.aconv
             del bin.ares
+        elif hasattr(bin, "alpha"):
+            for elt in [bin.csp, bin.queue, bin.alpha]:
+                elt.set_state(gst.STATE_NULL)
+                bin.remove(elt)
+            del bin.queue
+            del bin.csp
+            del bin.alpha
 
         if hasattr(bin, "ghostpad"):
             # singledecodebin found something on this pad
@@ -334,13 +359,16 @@ class SourceFactory(ObjectFactory):
             bin.remove_pad(bin.ghostpad)
             del bin.ghostpad
 
-    def _makeStreamBin(self, output_stream):
+    def _makeStreamBin(self, output_stream, child_bin=None):
         self.debug("output_stream:%r", output_stream)
         b = gst.Bin()
         b.decodebin = self.singleDecodeBinClass(uri=self.uri, caps=output_stream.caps,
                                            stream=output_stream)
         b.decodebin.connect("pad-added", self._singlePadAddedCb, b)
         b.decodebin.connect("pad-removed", self._singlePadRemovedCb, b)
+        if child_bin:
+            b.child = child_bin
+            b.add(child_bin)
 
         if isinstance(output_stream, AudioStream):
             self.debug("Adding volume element")
@@ -349,16 +377,62 @@ class SourceFactory(ObjectFactory):
             b.ares = gst.element_factory_make("audioresample", "internal-audioresample")
             b.volume = gst.element_factory_make("volume", "internal-volume")
             b.add(b.volume, b.ares, b.aconv)
-            gst.element_link_many(b.aconv, b.ares, b.volume)
+            if child_bin:
+                gst.element_link_many(b.aconv, b.ares, b.child, b.volume)
+                b.child.sync_state_with_parent()
+            else:
+                gst.element_link_many(b.aconv, b.ares, b.volume)
+                
             b.aconv.sync_state_with_parent()
             b.ares.sync_state_with_parent()
             b.volume.sync_state_with_parent()
+        elif isinstance(output_stream, VideoStream):
+            self.debug("Adding alpha element")
+            b.queue = gst.element_factory_make("queue", "internal-queue")
+            b.queue.props.max_size_bytes = 0
+            b.queue.props.max_size_time = 0
+            b.queue.props.max_size_buffers = 3
+
+            # all video needs to be AYUV, but the colorspace conversion
+            # element depends on the input. if there is no alpha we need to
+            # add ffmpegcolorspace. if we have an argb or rgba stream, we need
+            # alphacolor to preserve the alpha channel (ffmpeg clobbers it).
+            # if we have an ayuv stream we don't want any colorspace
+            # converter.
+
+            if not output_stream.has_alpha(): 
+                b.csp = gst.element_factory_make("ffmpegcolorspace",
+                    "internal-colorspace") 
+            elif output_stream.videotype == 'video/x-raw-rgb': 
+                b.csp = gst.element_factory_make("alphacolor", 
+                    "internal-alphacolor")
+            else: 
+                b.csp = gst.element_factory_make("identity")
+
+            b.alpha = gst.element_factory_make("alpha", "internal-alpha")
+            b.scale = SmartVideoScale()
+            b.scale.set_caps(self._filtercaps)
+            b.scale._computeAndSetValues()
+
+            b.add(b.queue, b.scale, b.csp, b.alpha)
+            gst.element_link_many(b.queue, b.csp, b.scale)
+            if child_bin:
+                gst.element_link_many(b.scale, b.child, b.alpha)
+                b.child.sync_state_with_parent()
+            else:
+                gst.element_link_many(b.scale, b.alpha)
+            b.scale.sync_state_with_parent()
+            b.queue.sync_state_with_parent()
+            b.csp.sync_state_with_parent()
+            b.alpha.sync_state_with_parent()
 
         b.add(b.decodebin)
         return b
 
     def _singlePadAddedCb(self, dbin, pad, topbin):
         self.debug("dbin:%r, pad:%r, topbin:%r", dbin, pad, topbin)
+        if hasattr(topbin, "child"):
+            topbin.child.sync_state_with_parent()
         if hasattr(topbin, "volume"):
             # make sure audio elements reach our same state. This is needed
             # since those elements are still unlinked downstream at this point,
@@ -369,6 +443,12 @@ class SourceFactory(ObjectFactory):
 
             pad.link(topbin.aconv.get_pad("sink"))
             topbin.ghostpad = gst.GhostPad("src", topbin.volume.get_pad("src"))
+        elif hasattr(topbin, "alpha"):
+            for element in [topbin.queue, topbin.scale, topbin.csp, topbin.alpha]:
+                element.sync_state_with_parent()
+
+            pad.link(topbin.queue.get_pad("sink"))
+            topbin.ghostpad = gst.GhostPad("src", topbin.alpha.get_pad("src"))
         else:
             topbin.ghostpad = gst.GhostPad("src", pad)
 
@@ -379,14 +459,30 @@ class SourceFactory(ObjectFactory):
 
     def _singlePadRemovedCb(self, dbin, pad, topbin):
         self.debug("dbin:%r, pad:%r, topbin:%r", dbin, pad, topbin)
-        topbin.remove_pad(topbin.ghostpad)
-        del topbin.ghostpad
+
+        # work around for http://bugzilla.gnome.org/show_bug.cgi?id=590735
+        if hasattr(topbin, "ghostpad"):
+            die = gst.Pad("die", gst.PAD_SRC)
+            topbin.ghostpad.set_target(die)
+
+            topbin.remove_pad(topbin.ghostpad)
+            del topbin.ghostpad
+
         if hasattr(topbin, "volume"):
             pad.unlink(topbin.aconv.get_pad("sink"))
+        elif hasattr(topbin, "alpha"):
+            pad.unlink(topbin.queue.get_pad("sink"))
 
     def addInputStream(self, stream):
         raise AssertionError("source factories can't have input streams")
 
+    def setFilterCaps(self, caps):
+        self._filtercaps = caps
+        for b in self.bins:
+            if hasattr(b, "scale"):
+                b.scale.set_caps(caps)
+                b.scale._computeAndSetValues()
+
 class SinkFactory(ObjectFactory):
     """
     Base class for factories that consume input and have no output.
