@@ -24,6 +24,7 @@ Base Formatter classes
 """
 
 import os
+from urlparse import urlparse
 from pitivi.project import Project
 from pitivi.utils import uri_is_reachable, uri_is_valid
 from pitivi.signalinterface import Signallable
@@ -68,7 +69,7 @@ class Formatter(Signallable, Loggable):
         "new-project-failed": ["uri", "exception"],
         "save-project-failed": ["project", "uri", "exception"],
         "project-saved": ["project", "uri"],
-        "missing-uri" : ["uri"]
+        "missing-uri" : ["uri", "factory"]
         }
 
     description = "Description of the format"
@@ -119,6 +120,7 @@ class Formatter(Signallable, Loggable):
         project = self.newProject()
         self.emit("new-project-created", project)
 
+        project.name = self._projectNameFromURI(location)
         project.uri = location
 
         self.log("location:%s, project:%r", location, project)
@@ -128,6 +130,11 @@ class Formatter(Signallable, Loggable):
         # FIXME : maybe have a convenience method for opening a location
         self._loadProject(location, project)
 
+    def _projectNameFromURI(self, uri):
+        path = urlparse(uri).path
+        basename = os.path.basename(path)
+        return os.path.splitext(basename)[0]
+
     def _finishLoadingProject(self, project):
         self.debug("About to get used sources")
         # ask for all sources being used
@@ -187,7 +194,10 @@ class Formatter(Signallable, Loggable):
             raise FormatterURIError()
         if overwrite == False and uri_is_reachable(location):
             raise FormatterOverwriteError()
+        old_uri = project.uri
         if self._saveProject(project, location):
+            project.uri = location
+            project.name = self._projectNameFromURI(location)
             self.emit("project-saved", project, location)
             return True
 
@@ -312,7 +322,7 @@ class Formatter(Signallable, Loggable):
 
         self.directorymapping[oldprefix] = newprefix
 
-    def validateSourceURI(self, uri):
+    def validateSourceURI(self, uri, factory):
         """
         Makes sure the given uri is accessible for reading.
 
@@ -329,12 +339,11 @@ class Formatter(Signallable, Loggable):
         self.debug("uri:%s", uri)
         if not uri_is_valid(uri):
             self.warning("invalid URI")
-            raise FormatterError("invalid URI")
+            raise FormatterError("invalid URI", uri)
 
         # skip non local uri
         if not uri.split('://', 1)[0] in ["file"]:
-            self.warning("We can only handle file:// URI")
-            return uri
+            raise FormatterError("We can only handle file:// URI", uri)
 
         # first check the good old way
         if uri_is_valid(uri) and uri_is_reachable(uri):
@@ -351,7 +360,7 @@ class Formatter(Signallable, Loggable):
                     return probable
 
         # else, let's fire the signal...
-        self.emit('missing-uri', uri)
+        self.emit('missing-uri', uri, factory)
 
         # and check again
         for k, v in self.directorymapping.iteritems():
