@@ -258,7 +258,8 @@ class ElementTreeFormatter(Formatter):
     def _deserializeDict(self, element):
         d = {}
         for a, b in element.attrib.iteritems():
-            d[a] = self._parsePropertyValue(b)
+            if len(b):
+                d[a] = self._parsePropertyValue(b)
         return d
 
     def _saveProjectSettings(self, settings):
@@ -368,7 +369,8 @@ class ElementTreeFormatter(Formatter):
 
     def _saveInterpolator(self, interpolator, prop):
         typename = prop.value_type.name
-        element = Element("curve", property=prop.name, type=typename)
+        element = Element("curve", property=prop.name, type=typename,
+            version="1")
 
         start = self._saveKeyframe(interpolator.start, typename, False)
         start.tag = "start"
@@ -407,6 +409,24 @@ class ElementTreeFormatter(Formatter):
         interpolator.end.value = self._parsePropertyValue(end.attrib["value"])
         interpolator.end.mode = int(end.attrib["mode"])
 
+        # if we are using old-style keyframe curves where start, end point
+        # represent start of file, convert to the newer representation to
+        # preserve the existing shape of the curve
+        if not ("version" in element.attrib):
+            # move start, end keyframes to start of file
+            interpolator.updateMediaStart(0)
+            interpolator.updateMediaStop(trackobject.factory.duration)
+
+            # get the value of the curve at true start/end points
+            startval = interpolator.valueAt(trackobject.in_point)
+            endval = interpolator.valueAt(trackobject.out_point)
+            interpolator.start.value = startval
+            interpolator.end.value = endval
+
+            # move start, end keyframes back to proper position
+            interpolator.updateMediaStart(trackobject.in_point)
+            interpolator.updateMediaStop(trackobject.out_point)
+
     def _saveTrackObjectRef(self, track_object):
         element = Element("track-object-ref")
         element.attrib["id"] = self._context.track_objects[track_object].attrib["id"]
@@ -442,8 +462,6 @@ class ElementTreeFormatter(Formatter):
         track_objects = SubElement(element, "track-objects")
 
         for track_object in track.track_objects:
-            if track_object is track.default_track_object:
-                continue
 
             track_object_element = self._saveTrackObject(track_object)
             track_objects.append(track_object_element)
