@@ -33,6 +33,8 @@ from pitivi.sourcelist import SourceList
 from pitivi.settings import ExportSettings
 from pitivi.signalinterface import Signallable
 from pitivi.action import ViewAction
+from pitivi.utils import Seeker
+import gst
 
 class ProjectError(Exception):
     """Project error"""
@@ -63,7 +65,7 @@ class Project(Signallable, Loggable):
     """
 
     __signals__ = {
-        "settings-changed" : None,
+        "settings-changed" : ['old', 'new'],
         }
 
     def __init__(self, name="", uri=None, **kwargs):
@@ -80,9 +82,9 @@ class Project(Signallable, Loggable):
         self.urichanged = False
         self.format = None
         self.sources = SourceList()
+        self.sources.connect("source-added", self._sourceAddedCb)
         self.sources.connect("source-removed", self._sourceRemovedCb)
 
-        self.settingssigid = 0
         self._dirty = False
 
         self.timeline = Timeline()
@@ -91,6 +93,21 @@ class Project(Signallable, Loggable):
         self.pipeline = Pipeline()
         self.view_action = ViewAction()
         self.view_action.addProducers(self.factory)
+        self.seeker = Seeker(80)
+
+        self.getCapsFromSettings()
+
+    def getCapsFromSettings(self):
+        settings = self.getSettings()
+        #formatstr = "video/x-raw-rgb,width=(int)%d,height=(int)%d;"\
+        #    "video/x-raw-yuv,width=(int)%d,height=(int)%d"
+        #capstr = formatstr % (
+        #    settings.videowidth,
+        #    settings.videoheight,
+        #    settings.videowidth,
+        #    settings.videoheight)
+        #self._videocaps = gst.Caps(capstr)
+        self._videocaps = settings.getVideoCaps()
 
     def release(self):
         self.pipeline.release()
@@ -98,9 +115,6 @@ class Project(Signallable, Loggable):
 
     #{ Settings methods
 
-    def _settingsChangedCb(self, unused_settings):
-        self.emit('settings-changed')
-
     def getSettings(self):
         """
         return the currently configured settings.
@@ -116,12 +130,10 @@ class Project(Signallable, Loggable):
         If settings is None, the current settings will be unset
         """
         self.log("Setting %s as the project's settings", settings)
-        if self.settings:
-            self.settings.disconnect(self.settingssigid)
+        oldsettings = self.settings
         self.settings = settings
-        self.emit('settings-changed')
-        self.settingssigid = self.settings.connect('settings-changed',
-            self._settingsChangedCb)
+        self._projectSettingsChanged()
+        self.emit('settings-changed', oldsettings, settings)
 
     def unsetSettings(self, unused_settings):
         """ Remove the currently configured settings."""
@@ -191,5 +203,16 @@ class Project(Signallable, Loggable):
     def hasUnsavedModifications(self):
         return self._dirty
 
+    def _projectSettingsChanged(self):
+        self.getCapsFromSettings()
+        for fact in self.sources.getSources():
+            fact.setFilterCaps(self._videocaps)
+        if self.pipeline.getState() != gst.STATE_NULL:
+            self.pipeline.stop()
+            self.pipeline.pause()
+
+    def _sourceAddedCb(self, sourcelist, factory):
+        factory.setFilterCaps(self._videocaps)
+
     def _sourceRemovedCb(self, sourclist, uri, factory):
         self.timeline.removeFactory(factory)
