@@ -38,6 +38,8 @@ UNSELECT = 1
 """Remove the given set from the selection."""
 SELECT_ADD = 2
 """Extend the selection with the given set"""
+SELECT_BETWEEN = 3
+"""Select a range of clips"""
 
 class TimelineError(Exception):
     """Base Exception for errors happening in L{Timeline}s or L{TimelineObject}s"""
@@ -457,6 +459,7 @@ class Selection(Signallable):
 
     def __init__(self):
         self.selected = set([])
+        self.last_single_obj = None
 
     def setToObj(self, obj, mode):
         """
@@ -480,7 +483,7 @@ class Selection(Signallable):
 
     # FIXME : it took me 10 mins to understand what this method does... a more obvious
     # name would be better :)
-    def setTo(self, selection, mode):
+    def setTo(self, objs, mode):
         """
         Update the current selection.
 
@@ -494,8 +497,13 @@ class Selection(Signallable):
 
         @see: L{setToObj}
         """
-        # get the L{TrackObject}s for the given TimelineObjects
-        selection = set([obj.timeline_object for obj in selection])
+        # get a list of timeline objects
+        selection = set()
+        for obj in objs:
+            if isinstance(obj, TrackObject):
+                selection.add(obj.timeline_object)
+            else:
+                selection.add(obj)
         old_selection = self.selected
         if mode == SELECT_ADD:
             selection = self.selected | selection
@@ -503,6 +511,9 @@ class Selection(Signallable):
             selection = self.selected - selection
         self.selected = selection
 
+        if len(self.selected) == 1:
+            self.last_single_obj = iter(selection).next()
+
         for obj in self.selected - old_selection:
             obj.selected = True
         for obj in old_selection - self.selected:
@@ -1128,8 +1139,6 @@ class MoveContext(EditingContext):
         self._restoreValues(self.default_originals)
 
     def finish(self):
-        EditingContext.finish(self)
-
         if isinstance(self.focus, TrackObject):
             focus_timeline_object = self.focus.timeline_object
         else:
@@ -1164,6 +1173,7 @@ class MoveContext(EditingContext):
                 break
 
         if not overlap:
+            EditingContext.finish(self)
             return
 
         self._defaultTo(initial_position, priority)
@@ -1176,6 +1186,7 @@ class MoveContext(EditingContext):
             final_position = initial_position - left_gap.duration
 
         self._defaultTo(final_position, priority)
+        EditingContext.finish(self)
 
     def _defaultTo(self, position, priority):
         if self._snap:
@@ -1249,7 +1260,7 @@ class TrimStartContext(EditingContext):
         self.default_originals = self._saveValues([focus_timeline_object])
 
         ripple = self.timeline.getObjsBeforeTime(focus.start)
-        assert not focus.timeline_object in ripple
+        assert not focus.timeline_object in ripple or focus.duration == 0
         self.ripple_originals = self._saveValues(ripple)
         self.ripple_offsets = self._getOffsets(focus.start, focus.priority,
             ripple)
@@ -1294,8 +1305,6 @@ class TrimStartContext(EditingContext):
         return position, priority
 
     def finish(self):
-        EditingContext.finish(self)
-
         initial_position = self.default_originals[self.focus_timeline_object][0]
 
         timeline_objects = [self.focus_timeline_object]
@@ -1307,6 +1316,7 @@ class TrimStartContext(EditingContext):
             left_gap, right_gap = Gap.findAroundObject(self.focus_timeline_object)
             position = initial_position - left_gap.duration
             self._defaultTo(position, self.focus.priority)
+        EditingContext.finish(self)
 
 class TrimEndContext(EditingContext):
 
@@ -1664,6 +1674,19 @@ class Timeline(Signallable, Loggable):
 
         @see: L{Selection.setToObj}
         """
+        if mode == SELECT_BETWEEN:
+            if self.selection.last_single_obj:
+                last = self.selection.last_single_obj
+                earliest = min(last.start, obj.start)
+                latest = max(last.start + last.duration,
+                    obj.start + obj.duration)
+                min_priority = min(last.priority, obj.priority)
+                max_priority = max(last.priority, obj.priority)
+                objs = self.getObjsInRegion(earliest, latest,
+                    min_priority, max_priority)
+                self.setSelectionTo(objs, SELECT)
+                return
+
         self.selection.setToObj(obj, mode)
 
     def setSelectionTo(self, selection, mode):
@@ -1752,13 +1775,13 @@ class Timeline(Signallable, Loggable):
             self.selection.setTo(timeline_object.track_objects, UNSELECT)
 
             for track_object in list(timeline_object.track_objects):
-                timeline_object.removeTrackObject(track_object)
-                new_timeline_object = TimelineObject(track_object.factory)
-                new_timeline_object.addTrackObject(track_object)
+                new_track_object = track_object.copy()
+                new_timeline_object = TimelineObject(new_track_object.factory)
+                new_timeline_object.addTrackObject(new_track_object)
                 self.addTimelineObject(new_timeline_object)
                 new_track_objects.extend(new_timeline_object.track_objects)
 
-            self.removeTimelineObject(timeline_object)
+            self.removeTimelineObject(timeline_object, deep=True)
 
         self.selection.setTo(new_track_objects, SELECT_ADD)
 
@@ -1771,6 +1794,17 @@ class Timeline(Signallable, Loggable):
             self.removeTimelineObject(timeline_object, deep=True)
         self.selection.setTo(set([]), SELECT)
 
+    def split(self, time):
+        """
+        Splits objects under the playehad. If the selection is not empty, the
+        split only applies to selected clips. Otherwise it applies to all
+        clips"""
+        objs = set(self.getObjsAtTime(time))
+        if len(self.selection):
+            objs = self.selection.selected.intersection(objs)
+        for obj in objs:
+            obj.split(time)
+
     def rebuildEdges(self):
         self.edges = TimelineEdges()
         for timeline_object in self.timeline_objects:
@@ -1815,6 +1849,12 @@ class Timeline(Signallable, Loggable):
 
         self.emit("disable-updates", False)
 
+    def getObjsAtTime(self, time_):
+        objects = self.timeline_objects
+        return [obj for obj in objects if
+            (obj.start < time_) and 
+            ((obj.start + obj.duration) > time_)]
+
     def getObjsAfterObj(self, obj):
         return self.getObjsAfterTime(obj.start + obj.duration)
 
@@ -1828,3 +1868,15 @@ class Timeline(Signallable, Loggable):
     def getObjsBeforeTime(self, target):
         return [to for to in self.timeline_objects 
             if to.start + to.duration <=target]
+
+    def getObjsInRegion(self, start, end, min_priority=0,
+        max_priority=4294967295L):
+        objs = []
+        for to in self.timeline_objects:
+            if ((to.start >= start) and
+                ((to.start + to.duration) <= end) and
+                (to.priority >= min_priority) and
+                (to.priority <= max_priority)):
+                objs.append(to)
+        return objs
+
