@@ -29,7 +29,7 @@ from pitivi.log.loggable import Loggable
 from pitivi.encode import encoders_muxer_compatible, muxer_can_sink_raw_audio, muxer_can_sink_raw_video
 from glade import GladeWidget
 from gstwidget import GstElementSettingsDialog
-
+import gobject
 from gettext import gettext as _
 
 class ExportSettingsWidget(GladeWidget, Loggable):
@@ -43,13 +43,17 @@ class ExportSettingsWidget(GladeWidget, Loggable):
                     (_("SVGA (800x600)"), 800, 600, 30.0, 1.0),
                     (_("XGA (1024x768)"), 1024, 768, 30.0, 1.0),
                     )
-    video_rates = ( ("12 fps",      12.0, 1.0),
-                    ("24 fps",      24.0, 1.0),
-                    ("23,97 fps",   24000.0, 1001.0),
-                    ("25 fps",      25.0, 1.0),
-                    ("29,97 fps",   30000.0, 1001.0),
-                    ("30 fps",      30.0, 1.0),
-                    ("60 fps",      60.0, 1.0) )
+    video_rates = ( ("12 fps", 12.0, 1.0),
+                    ("15 fps", 15.0, 1.0),
+                    ("20 fps", 20.0, 1.0),
+                    ("23,976 fps", 24000.0, 1001.0),
+                    ("24 fps", 24.0, 1.0),
+                    ("25 fps", 25.0, 1.0),
+                    ("29,97 fps", 30000.0, 1001.0),
+                    ("30 fps", 30.0, 1.0),
+                    ("59,94 fps", 60000.0, 1001.0),
+                    ("60 fps", 60.0, 1.0),
+                    ("120 fps", 120.0, 1.0) )
     audio_presets = ( ("CD",  2,      44100,  16), )
     audio_rates = ( ("8 KHz",   8000),
                     ("11 KHz",  11025),
@@ -63,7 +67,7 @@ class ExportSettingsWidget(GladeWidget, Loggable):
                      ("32 bit", 32) )
 
 
-    def __init__(self):
+    def __init__(self, app):
         GladeWidget.__init__(self)
         Loggable.__init__(self)
         self.settings = None
@@ -76,6 +80,7 @@ class ExportSettingsWidget(GladeWidget, Loggable):
         self.muxer = None
         self.vencoder = None
         self.aencoder = None
+        self.app = app
         self._loading = False
 
     def setSettings(self, settings):
@@ -314,7 +319,22 @@ class ExportSettingsWidget(GladeWidget, Loggable):
         self.acodeccbox.set_active(selected)
 
     def runSettingsDialog(self, factory, settings={}):
+        def configureEventCb(window, event):
+            self.app.settings.elementSettingsDialogWidth = event.width
+            self.app.settings.elementSettingsDialogHeight = event.height
+
+        def mapEventCb(window, event):
+            def reallySetWindowSize():
+                dialog.window.resize(self.app.settings.elementSettingsDialogWidth,
+                    self.app.settings.elementSettingsDialogHeight)
+                dialog.window.disconnect(mapEventId)
+                window.connect("configure-event", configureEventCb)
+                return False
+            gobject.idle_add(reallySetWindowSize)
+
         dialog = GstElementSettingsDialog(factory, settings)
+        mapEventId = dialog.window.connect("map-event", mapEventCb)
+
         if dialog.run() == gtk.RESPONSE_OK:
             dialog.hide()
             settings = dialog.getSettings()
@@ -414,13 +434,14 @@ class ExportSettingsWidget(GladeWidget, Loggable):
 
 class ExportSettingsDialog(gtk.Dialog):
 
-    def __init__(self, settings):
+    def __init__(self, app, settings):
         gtk.Dialog.__init__(self, parent=None,
                             title=_("Export settings"),
                             flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-        self.setwidget = ExportSettingsWidget()
+        self.set_icon_name("pitivi")
+        self.setwidget = ExportSettingsWidget(app)
         self.vbox.pack_start(self.setwidget)
         self.setwidget.setSettings(settings)
         self.setwidget.show_all()
