@@ -40,10 +40,11 @@ from zoominterface import Zoomable
 from pitivi.ui.common import LAYER_HEIGHT_EXPANDED, LAYER_SPACING
 from pitivi.timeline.timeline import MoveContext
 from pitivi.utils import Seeker
+from pitivi.ui.filelisterrordialog import FileListErrorDialog
 
 # tooltip text for toolbar
 DELETE = _("Delete Selected")
-RAZOR = _("Cut clip at mouse position")
+SPLIT = _("Split clip at playhead position")
 ZOOM_IN =  _("Zoom In")
 ZOOM_OUT =  _("Zoom Out")
 UNLINK = _("Break links between clips")
@@ -64,7 +65,7 @@ ui = '''
         </menu>
         <menu action="Timeline">
             <placeholder name="Timeline">
-                <menuitem action="Razor" />
+                <menuitem action="Split" />
                 <separator />
                 <menuitem action="DeleteObj" />
                 <menuitem action="LinkObj" />
@@ -76,10 +77,8 @@ ui = '''
     </menubar>
     <toolbar name="TimelineToolBar">
         <placeholder name="Timeline">
-            <toolitem action="ZoomOut" />
-            <toolitem action="ZoomIn" />
             <separator />
-            <toolitem action="Razor" />
+            <toolitem action="Split" />
             <separator />
             <toolitem action="DeleteObj" />
             <toolitem action="UnlinkObj" />
@@ -111,6 +110,73 @@ ui = '''
 #    |
 #    +--Status Bar ??
 
+class InfoStub(gtk.HBox, Loggable):
+    """
+    Box used to display information on the current state of the timeline
+    """
+
+    def __init__(self):
+        gtk.HBox.__init__(self)
+        Loggable.__init__(self)
+        self.errors = []
+        self.showing = False
+        self._errorsmessage = _("One or more GStreamer errors has occured!")
+        self._makeUI()
+
+    def _makeUI(self):
+        self.set_spacing(6)
+        self.erroricon = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
+                                                  gtk.ICON_SIZE_SMALL_TOOLBAR)
+
+        self.pack_start(self.erroricon, expand=False)
+
+
+        self.infolabel = gtk.Label(self._errorsmessage)
+        self.infolabel.set_alignment(0, 0.5)
+
+        self.questionbutton = gtk.Button()
+        self.questionbutton.set_image(gtk.image_new_from_stock(gtk.STOCK_INFO,
+                                                               gtk.ICON_SIZE_SMALL_TOOLBAR))
+        self.questionbutton.connect("clicked", self._questionButtonClickedCb)
+        self._questionshowing = False
+
+        self.pack_start(self.infolabel, expand=True, fill=True)
+        self.pack_start(self.questionbutton, expand=False)
+
+    def addErrors(self, *args):
+        self.errors.append(args)
+        self.show()
+
+    def _errorDialogBoxCloseCb(self, dialog):
+        dialog.destroy()
+
+    def _errorDialogBoxResponseCb(self, dialog, unused_response):
+        dialog.destroy()
+
+    def _questionButtonClickedCb(self, unused_button):
+        msgs = (_("Error List"),
+            _("The following errors have been reported:"))
+        # show error dialog
+        dbox = FileListErrorDialog(*msgs)
+        dbox.connect("close", self._errorDialogBoxCloseCb)
+        dbox.connect("response", self._errorDialogBoxResponseCb)
+        for reason, extra in self.errors:
+            dbox.addFailedFile(None, reason, extra)
+        dbox.show()
+        # reset error list
+        self.errors = []
+        self.hide()
+
+    def show(self):
+        self.log("showing")
+        self.show_all()
+        self.showing = True
+
+    def hide(self):
+        self.log("hiding")
+        gtk.VBox.hide(self)
+        self.showing = False
+
 class Timeline(gtk.Table, Loggable, Zoomable):
 
     # the screen width of the current unit
@@ -125,6 +191,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         Zoomable.__init__(self)
         self.log("Creating Timeline")
 
+        self._updateZoom = True
         self.project = None
         self.ui_manager = ui_manager
         self.app = instance
@@ -136,52 +203,67 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._createUI()
         self._prev_duration = 0
         self.shrink = True
-        self._seeker = Seeker(80)
-        self._seeker.connect('seek', self._seekerSeekCb)
+        self.rate = gst.Fraction(1,1)
 
     def _createUI(self):
         self.leftSizeGroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        self.props.row_spacing = 2
+        self.props.column_spacing = 2
         self.hadj = gtk.Adjustment()
         self.vadj = gtk.Adjustment()
 
+        # zooming slider
+        self._zoomAdjustment = gtk.Adjustment()
+        self._zoomAdjustment.set_value(Zoomable.getCurrentZoomLevel())
+        self._zoomAdjustment.connect("value-changed",
+            self._zoomAdjustmentChangedCb)
+        self._zoomAdjustment.props.lower = 0
+        self._zoomAdjustment.props.upper = Zoomable.zoom_steps
+        zoomslider = gtk.HScale(self._zoomAdjustment)
+        zoomslider.props.draw_value = False
+        zoomslider.set_tooltip_text(_("Zoom Timeline"))
+        self.attach(zoomslider, 0, 1, 0, 1, yoptions=0, xoptions=gtk.FILL)
+
         # controls for tracks and layers
         self._controls = TimelineControls()
-        controlwindow = gtk.ScrolledWindow(None, self.vadj)
-        controlwindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
-        controlwindow.add_with_viewport(self._controls)
+        controlwindow = gtk.Viewport(None, self.vadj)
+        controlwindow.add(self._controls)
         controlwindow.set_size_request(-1, 1)
+        controlwindow.set_shadow_type(gtk.SHADOW_OUT)
         self.attach(controlwindow, 0, 1, 1, 2, xoptions=0)
 
         # timeline ruler
-        self.ruler = ruler.ScaleRuler(self.hadj)
-        self.ruler.set_size_request(0, 35)
+        self.ruler = ruler.ScaleRuler(self.app, self.hadj)
+        self.ruler.set_size_request(0, 25)
         self.ruler.set_border_width(2)
         self.ruler.connect("key-press-event", self._keyPressEventCb)
-        self.attach(self.ruler, 1, 2, 0, 1, yoptions=0)
+        self.ruler.connect("size-allocate", self._rulerSizeAllocateCb)
+        rulerframe = gtk.Frame()
+        rulerframe.set_shadow_type(gtk.SHADOW_OUT)
+        rulerframe.add(self.ruler)
+        self.attach(rulerframe, 1, 2, 0, 1, yoptions=0)
 
         # proportional timeline
         self._canvas = TimelineCanvas(self.app)
         timelinewindow = gtk.ScrolledWindow(self.hadj, self.vadj)
         timelinewindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         timelinewindow.add(self._canvas)
+        timelinewindow.set_shadow_type(gtk.SHADOW_IN)
         #FIXME: remove padding between scrollbar and scrolled window
         self.attach(timelinewindow, 1, 2, 1, 2)
 
-        # drag and drop
-        self.drag_dest_set(gtk.DEST_DEFAULT_MOTION,
-            [dnd.FILESOURCE_TUPLE],
-            gtk.gdk.ACTION_COPY)
+        # error infostub
+        self.infostub = InfoStub()
+        self.attach(self.infostub, 1, 2, 2, 3, yoptions=0)
 
-        self.connect("drag-data-received", self._dragDataReceivedCb)
-        self.connect("drag-leave", self._dragLeaveCb)
-        self.connect("drag-drop", self._dragDropCb)
-        self.connect("drag-motion", self._dragMotionCb)
+        self.show_all()
+        self.infostub.hide()
 
         # toolbar actions
         actions = (
-            ("ZoomIn", gtk.STOCK_ZOOM_IN, None, None, ZOOM_IN,
+            ("ZoomIn", gtk.STOCK_ZOOM_IN, None, "<Control>equal", ZOOM_IN,
                 self._zoomInCb),
-            ("ZoomOut", gtk.STOCK_ZOOM_OUT, None, None, ZOOM_OUT,
+            ("ZoomOut", gtk.STOCK_ZOOM_OUT, None, "<Control>minus", ZOOM_OUT,
                 self._zoomOutCb),
         )
 
@@ -198,14 +280,14 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                 self.groupSelected),
         )
 
-        toggle_actions = (
-            ("Razor", "pitivi-split", _("Razor"), "<Ctrl>R", RAZOR,
-                self.toggleRazor),
+        playhead_actions = (
+            ("Split", "pitivi-split", _("Split"), "S", SPLIT,
+                self.split),
         )
 
         actiongroup = gtk.ActionGroup("timelinepermanent")
         actiongroup.add_actions(actions)
-        actiongroup.add_toggle_actions(toggle_actions)
+        actiongroup.add_actions(playhead_actions)
         self.ui_manager.insert_action_group(actiongroup, 0)
 
         actiongroup = gtk.ActionGroup("timelineselection")
@@ -270,9 +352,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             pipeline.getDuration()))
         self._seeker.seek(seekvalue)
 
-    def _seekerSeekCb(self, seeker, position, format):
-        self.project.pipeline.seek(position, format)
-
     def _buttonPress(self, window, event):
         self.shrink = False
 
@@ -324,6 +403,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         context.drop_finish(True, timestamp)
         self._factories = None
         self._temp_objects = None
+        self.app.current.seeker.seek(self._position)
         return True
 
     def _dragDataReceivedCb(self, unused_layout, context, x, y,
@@ -337,10 +417,11 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         # tell current project to import the uri
         # wait for source-added signal, meanwhile ignore dragMotion signals
         # when ready, add factories to the timeline.
-        if targetType == dnd.TYPE_PITIVI_FILESOURCE:
-            uris = selection.data.split("\n")
-        else:
+        if targetType != dnd.TYPE_PITIVI_FILESOURCE:
             context.finish(False, False, timestamp)
+            return
+
+        uris = selection.data.split("\n")
         self._factories = [self.project.sources.getUri(uri) for uri in uris]
         context.drag_status(gtk.gdk.ACTION_COPY, timestamp)
         return True
@@ -359,15 +440,23 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
 ## Zooming and Scrolling
 
+    def _zoomAdjustmentChangedCb(self, adjustment):
+        # GTK crack
+        self._updateZoom = False
+        Zoomable.setZoomLevel(int(adjustment.get_value()))
+        self._updateZoom = True
+
     def zoomChanged(self):
-        # this has to be in a timeout, because the resize hasn't actually
-        # completed yet, and so the canvas can't actually complete the scroll
-        gobject.idle_add(self.scrollToPlayhead)
+        self._canvas.props.redraw_when_scrolled = True
+        if self._updateZoom:
+            self._zoomAdjustment.set_value(self.getCurrentZoomLevel())
+        self.ruler.queue_resize()
+        self.ruler.queue_draw()
 
     def timelinePositionChanged(self, position):
         self._position = position
         self.ruler.timelinePositionChanged(position)
-        self._canvas._position = position
+        self._canvas.timelinePositionChanged(position)
         if self._state == gst.STATE_PLAYING:
             self.scrollToPlayhead()
 
@@ -394,6 +483,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self.hadj.set_value(position)
         return False
 
+    def _rulerSizeAllocateCb(self, ruler, allocation):
+        self._canvas.props.redraw_when_scrolled = False
+
 ## Project callbacks
 
     def _setProject(self):
@@ -402,14 +494,18 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             self._controls.timeline = self.timeline
             self._canvas.timeline = self.timeline
             self._canvas.zoomChanged()
+            self.ruler.setProjectFrameRate(self.project.getSettings().videorate)
             self.ruler.zoomChanged()
-            self._settingsChangedCb(self.project)
+            self._settingsChangedCb(self.project, None, self.project.getSettings())
+            self._seeker = self.project.seeker
 
     project = receiver(_setProject)
 
     @handler(project, "settings-changed")
-    def _settingsChangedCb(self, project):
-        self.rate = float(1 / self.project.getSettings().videorate)
+    def _settingsChangedCb(self, project, old, new):
+        rate = new.videorate
+        self.rate = float(1 / rate)
+        self.ruler.setProjectFrameRate(rate)
 
 ## Timeline callbacks
 
@@ -480,9 +576,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self.actiongroup.set_visible(False)
         gtk.Vbox.hide(self)
 
-    def _computeZoomRatio(self, index):
-        return self.zoom_levels[index]
-
     def _zoomInCb(self, unused_action):
         Zoomable.zoomIn()
 
@@ -505,14 +598,17 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
     def ungroupSelected(self, unused_action):
         if self.timeline:
+            self.app.action_log.begin("ungroup")
             self.timeline.ungroupSelection()
+            self.app.action_log.commit()
 
     def groupSelected(self, unused_action):
         if self.timeline:
             self.timeline.groupSelection()
 
-    def toggleRazor(self, action):
-        if action.props.active:
-            self._canvas.activateRazor(action)
-        else:
-            self._canvas.deactivateRazor()
+    def split(self, action):
+        self.app.action_log.begin("split")
+        self.timeline.split(self._position)
+        self.app.action_log.commit()
+        # work-around for 603149
+        self.project.seeker.seek(self._position)
