@@ -272,6 +272,17 @@ class PitiviViewer(gtk.VBox, Loggable):
             width += 110
             height = int(width / self.aframe.props.ratio)
             self.aframe.set_size_request(width , height)
+        self.show_all()
+
+    _showingSlider = True
+
+    def showSlider(self):
+        self._showingSlider = True
+        self.slider.show()
+
+    def hideSlider(self):
+        self._showingSlider = False
+        self.slider.hide()
 
     def showControls(self):
         if not self.action:
@@ -282,7 +293,8 @@ class PitiviViewer(gtk.VBox, Loggable):
             self.playpause_button.show()
             self.next_button.show()
             self.forward_button.show()
-            self.slider.show()
+            if self._showingSlider:
+                self.slider.show()
         else:
             self.rewind_button.hide()
             self.back_button.hide()
@@ -306,14 +318,18 @@ class PitiviViewer(gtk.VBox, Loggable):
 
     ## gtk.HScale callbacks for self.slider
 
-    def _sliderButtonPressCb(self, slider, unused_event):
+    def _sliderButtonPressCb(self, slider, event):
+        # borrow totem hack for seek-on-click behavior
+        event.button = 2
         self.info("button pressed")
         self.moving_slider = True
         self.valuechangedid = slider.connect("value-changed", self._sliderValueChangedCb)
         self.pipeline.pause()
         return False
 
-    def _sliderButtonReleaseCb(self, slider, unused_event):
+    def _sliderButtonReleaseCb(self, slider, event):
+        # borrow totem hack for seek-on-click behavior
+        event.button = 2
         self.info("slider button release at %s", time_to_string(long(slider.get_value())))
         self.moving_slider = False
         if self.valuechangedid:
