@@ -1,9 +1,10 @@
 import unittest
 import pitivi
-from pitivi.pitivi import Pitivi
+from pitivi.application import Pitivi
 from pitivi.utils import binary_search
+from common import TestCase
 
-class BasicTest(unittest.TestCase):
+class BasicTest(TestCase):
     """
     Basic test to create the proper creation of the Pitivi object
     """
